/* -*-mode: C++; style: K&R; c-basic-offset: 4 ; -*- */

/**
 *  curl.cpp
 *   Simple wrapper around libCurl to download a remote URI simply.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  Steve Kemp
 *  ---
 *  http://www.steve.org.uk/
 *
 */


#include <stdlib.h>
#include <string.h>

#include "curl.h"


#include "version.h"
InstallVersion dummyCurl( __FILE__, __DATE__, "$Revision: 1.2 $" );


/**
 * Constructor.
 */
CCurl::CCurl()
{
    m_url = NULL;
    m_data = NULL;
    m_data_size = 0;
}


/**
 * Destructor.  Tidy up and free memory.
 */
CCurl::~CCurl()
{
    if ( m_url != NULL )
    {
        curl_free( m_url );
    }
    if ( m_data != NULL )
    {
        free( m_data );
    }
}


/**
 * Download a remote URI.
 */
void CCurl::fetchURI( const char *uri )
{
    if ( m_url == NULL )
    {
        m_url = curl_easy_init();
    }
    curl_easy_setopt( m_url, CURLOPT_URL, uri );
    curl_easy_setopt( m_url, CURLOPT_NOPROGRESS );
    curl_easy_setopt( m_url, CURLOPT_WRITEFUNCTION, write_data);
    curl_easy_setopt( m_url, CURLOPT_WRITEDATA, this);
    curl_easy_perform( m_url );
}


/**
 *  Find and return the status code from the previous request.
 * TODO: Assert we've downloaded, via Fetch.
 */
long CCurl::getCode()
{
    long code;
    curl_easy_getinfo( m_url, CURLINFO_RESPONSE_CODE, &code );
    return( code );
}


/**
 * Callback routine for libCurl.
 * @param ptr A pointer to the data chunk downloaded.
 * @param size The size of data we are being passed.
 * @param nmemb The size of each byte of data included in size.
 * @param userp A user-supplied pointer.
 * @returns The amount of data successfully managed (and added to our m_data buffer).
 */
/* static */ size_t CCurl::write_data(void *ptr, size_t size, size_t nmemb, void *userp)
{
  /*
   * Find ourself - this is a static function, remember?
   */
  CCurl *us = (CCurl*)userp;

  /* Size of data libCurl is giving us. */
  int len = size * nmemb;

  /* Resize our data buffer to allow the new data to fit into it. */
  us->m_data = realloc( us->m_data, us->m_data_size + len);

  /* Append the new data to the buffer. */
  memcpy( ((char *)us->m_data) + us->m_data_size, ptr, len);

  /* Keep track of the size. */
  us->m_data_size += len;

  return len;
}


/**
 * Return the downloaded data to the caller.
 * TODO: Assert we've downloaded, via Fetch.
 */
char *CCurl::getData()
{
  return( (char *)m_data );
}
