/* -*-mode: C++; style: K&R; c-basic-offset: 4 ; -*- */

/*
 *  portscanner.cc
 *   Simple wrapper around a naive TCP/IP port scanner.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  Steve Kemp
 *  ---
 *  http://www.steve.org.uk/
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <netdb.h>
#include <netinet/in.h>


#include "portscanner.h"
#include "version.h"

InstallVersion dummyPortscanner( __FILE__, __DATE__, "$Revision: 1.4 $" );



/**
 * Constructor for this object.
 * @param host The machine to scan.
 * @returns  A new CPortScanner object.
 */
CPortScanner::CPortScanner( const char *host )
{
    m_hostName = std::string( host );
}


/**
 * Destructor.
 */
CPortScanner::~CPortScanner()
{
}


/**
 * Connect to each port in turn and return an STL vector
 * of all open ports upon ths host.
 *
 * @param start The starting port to scan from.
 * @param end   The end port to scan until
 * @return The vector containing all open ports
 */
std::vector< int > CPortScanner::doScan( int start, int end )
{
    std::vector< int > results;

    struct sockaddr_in dest;
    struct hostent *he;

    if( ( he = gethostbyname( m_hostName.c_str() ) ) == NULL )
    {
	printf("Couldn't resolve %s\n", m_hostName.c_str() );
	return( results );
    }

    for(int count = start; count <= end; count++ )
    {
	int sock;

	/* Create socket */
	if((sock = socket(AF_INET, SOCK_STREAM, 0)) == -1)
	{
	    printf("Couldn't create socket!\n");
	    return( results );
	}

	/* Setup type + port */
	dest.sin_family = AF_INET;
	dest.sin_port = htons(count);
	dest.sin_addr = *((struct in_addr *)he->h_addr);

	/* Attempt to connect */
	if( connect( sock, (struct sockaddr *)&dest, sizeof(struct sockaddr) )
	    != -1 )
	{
	    /* Success: So we have an open port */
	    results.push_back( count );
	}

	/* Close socket */
	close(sock);
    }

    return( results );
}


