/* -*-mode: C++; style: K&R; c-basic-offset: 4 ; -*- */

/*
 *  portscanner.h
 *   Simple wrapper around a naive TCP/IP port scanner.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  Steve Kemp
 *  ---
 *  http://www.steve.org.uk/
 *
 */


#ifndef _PORT_SCANNER_H
#define _PORT_SCANNER_H 1


#include <vector>
#include <string>


/**
 * This is a simple TCP connection based port scanner which will attempt
 * to connect to all the privileged ports upon a given host.
 *
 * Privileged ports mean that the scanner will only attempt to connect
 * to ports 0-1024.
 *
 * Sample usage looks like this:
 *
 * @code
 *
 * CPortScanner object( "hostname" );
 * std::vector< int > openPorts = object.doScan();
 *
 * @endcode
 *
 * @version $Id: portscanner.h,v 1.2 2005/10/18 00:39:07 steve Exp $
 * @author Steve Kemp http://www.steve.org.uk/
 */
class CPortScanner
{

public:

   /**
     * Constructor for this object.
     * @param host The machine to scan.
     * @returns  A new CPortScanner object.
     */
   CPortScanner( const char *host );


   /**
     * Destructor.
     */
   ~CPortScanner( );


public:

    /**
     * Connect to each port in turn and return an STL vector
     * of all open ports upon ths host.
     *
     * @param start The starting port to scan from.
     * @param end   The end port to scan until
     * @return The vector containing all open ports
     */
    std::vector< int > doScan( int start = 0, int end = 1024 );


private:
    /**
     * The hostname we are going to scan.
     */
    std::string m_hostName;


};

#endif /* _PORT_SCANNER_H */
