/* -*-mode: C++; style: K&R; c-basic-offset: 4 ; -*- */

/*
 *  servicedetect.cc
 *   Simple class which attempts to detect the service running upon a port.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  Steve Kemp
 *  ---
 *  http://www.steve.org.uk/
 *
 */


#include <stdio.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>    /* sockaddr_in structure */
#include <netdb.h>

#include "servicedetect.h"

#include "version.h"
InstallVersion dummyServiceDetect( __FILE__, __DATE__, "$Revision: 1.3 $" );

/**
 * Constructor.
 * Connect to the named host and port and attempt to detect the
 * service running upon the host.
 * @param host The hostname to connect to.
 * @param port The port to connect to.
 */
CServiceDetect::CServiceDetect( char *host, int port )
{
    /* Zero our results buffer */
    snprintf( m_serviceName, sizeof( m_serviceName ), "%s", "connect-error" );

    /* Create socket */
    int sock = createSocket( host, port );
    
    /* Send greeting */
    char *text = "QUIT\n";
    write(sock, text, strlen(text));

    /* Read a temporary result */
    char buffer[128];
    int readLen = read(sock, buffer, sizeof(buffer));
    
    if ( readLen )
    {
        /* Close socket */
        close( sock );

	/* We connected, but we don't know what we got yet. */
	snprintf( m_serviceName, sizeof( m_serviceName ), "%s", "unknown" );

	/* Lowercase the input */
	buffer[readLen]='\0';
	for( int i = 0; i < readLen-1; i++ )
	{
	    buffer[i] = tolower(buffer[i]);
	}

	/* Now do the magic. */
	if ( strncmp( buffer, "ssh", 3 ) == 0 )
	    snprintf( m_serviceName, sizeof( m_serviceName ), "%s", "ssh" );
	else
	    if ( strncmp( buffer, "http", 4 ) == 0 )
	        snprintf( m_serviceName, sizeof( m_serviceName ), "%s", "www" );
	    else
	        if ( buffer[0] == '<' ) 
  		    snprintf( m_serviceName, sizeof( m_serviceName ), "%s", "www" );
		else
		    if ( strstr( buffer, "ftp" ) != NULL )
			snprintf( m_serviceName, sizeof( m_serviceName ), "%s", "ftp" );
	            else
			if ( strncmp( buffer, "220 ", 4 ) == 0 )
			    snprintf( m_serviceName, sizeof( m_serviceName ), "%s", "smtp" );
			else
			    strncat( m_serviceName, buffer, readLen );
    }

}


/**
 * Destructor.
 */
CServiceDetect::~CServiceDetect()
{
    /* 
     * NOP
     */
}


/**
 * Get service name.
 * @return The guessed service name.
 */
char *CServiceDetect::getServiceName( )
{
    return( m_serviceName );
}


int CServiceDetect::createSocket( char *hostname, int port )
{
    int s;
    struct sockaddr_in sa;
    struct hostent *hp;
    int ret;
 
    hp = gethostbyname(hostname);
    bcopy((char *)hp->h_addr, (char *)&sa.sin_addr, hp->h_length);
    sa.sin_family = hp->h_addrtype;
    sa.sin_port = htons(port);
    s = socket(hp->h_addrtype, SOCK_STREAM, 0);
    ret = connect(s, (struct sockaddr *)&sa, sizeof(sa));
    
    return s;
	
}
