/* -*-mode: C++; style: K&R; c-basic-offset: 4 ; -*- */

/*
 *  servicedetect.h
 *   Simple class which attempts to detect the service running upon a port.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  Steve Kemp
 *  ---
 *  http://www.steve.org.uk/
 *
 */


#ifndef _SERVICE_DETECT_H
#define _SERVICE_DETECT_H 1




/**
 * This is a simple wrapper class which connets to a host and port
 * and attempts to recognise the service upon the other end.
 *
 * @version $Id: servicedetect.h,v 1.2 2005/10/08 14:43:23 steve Exp $
 * @author Steve Kemp http://www.steve.org.uk/
 */
class CServiceDetect
{

public:


   /**
    * Constructor.
    * Connect to the named host and port and attempt to detect the
    * service running upon the host.
    * @param host The hostname to connect to.
    * @param port The port to connect to.
    */
    CServiceDetect( char *host, int port );


    /**
     * Destructor.
     */
    ~CServiceDetect();


public:

    /**
     * Get service name.
     * @return The guessed service name.
     */
    char *getServiceName( );


protected:


    /**
     * Create a socket connection to the given server and port.
     * @param hostname The hostname to connect to.
     * @param port The port number to connec tto.
     * @return The connected socket.
     */
    int createSocket( char *hostname, int port );


private:

    /**
     * Buffer to hold service name.
     */
    char m_serviceName[ 1024 ];


};

#endif /* _SERVICE_DETECT_H */
