/* -*-mode: C++; style: K&R; c-basic-offset: 4 ; -*- */

/*
 *  services.cc
 *   Simple wrapper around /etc/services
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  Steve Kemp
 *  ---
 *  http://www.steve.org.uk/
 *
 */


#include <netdb.h>

#include "services.h"

#include "version.h"
InstallVersion dummyServices( __FILE__, __DATE__, "$Revision: 1.2 $" );



/**
 * Lookup and return the primary name used for a given port.
 *
 * @param port The port to find the name(s) for.
 * @return The primary service name.
 */
/* static */ std::string CServices::LookupByPort( int port, const char *proto)

{
   
    struct servent *services = getservbyport( ( int) ntohs((port)) , proto );

    /*
     * TODO: Lookup ALIASes
     */
    if ( services == NULL )
    {
	return( "" );
    }
    return( std::string( services->s_name  ) );

}


/**
 * Lookup and return the primary port used for a given service
 *
 * @param name The service to find the port for.
 * @return The port used by the given service
 */
/* static */ int  CServices::LookupByName( const char *name, const char *proto )
{
    struct servent *services = getservbyname( name , proto );

    /*
     * TODO: Lookup ALIASes
     */
    if ( services == NULL )
    {
	return( 0 );
    }
    return( (int)ntohs(services->s_port ) );

}
