/* -*-mode: C++; style: K&R; c-basic-offset: 4 ; -*- */

/*
 *  services.h
 *   Simple wrapper around /etc/services
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  Steve Kemp
 *  ---
 *  http://www.steve.org.uk/
 *
 */


#ifndef _SERVICES_H
#define _SERVICES_H 1


#include <vector>
#include <string>



/**
 * This is a simple wrapper around the services file /etc/services upon
 * Unix hosts.
 *
 * @version $Id: services.h,v 1.2 2005/10/08 14:43:23 steve Exp $
 * @author Steve Kemp http://www.steve.org.uk/
 */
class CServices
{

public:


    /**
     * Lookup and return the primary name used for a given port.
     *
     * @param port The port to find the name(s) for.
     * @param proto The protocol to use.
     * @return The primary service name.
     */
    static std::string LookupByPort( int port, const char *proto = "tcp" );



    /**
     * Lookup and return the primary port used for a given service
     *
     * @param name The service to find the port for.
     * @param proto The protocol to use.
     * @return The port used by the given service
     */
    static int LookupByName( const char *name, const char *proto = "tcp" );


};

#endif /* _SERVICES_H */
